@@ -20,7 +20,7 @@ from zim.formats.wiki import Parser as WikiParser
 from zim.notebook import *
 from zim.notebook.notebook import NotebookConfig, IndexNotUptodateError, PageExistsError
 from zim.notebook.index import Index
-from zim.notebook.layout import FilesLayout
+from zim.notebook.layout import FilesLayout, FILE_TYPE_PAGE_SOURCE, FILE_TYPE_ATTACHMENT
 
 
 class TestNotebookInfo(tests.TestCase):
@@ -132,8 +132,24 @@ class TestNotebookInfoList(tests.TestCase):
 		self.assertEqual(interwiki_link('bar?Foo'), 'zim+' + uri1 + '?Foo') # name
 		self.assertEqual(interwiki_link('Bar?Foo'), 'zim+' + uri1 + '?Foo') # name
 
-		# Check backward compatibility
-		file = tests.TEST_DATA_FOLDER.file('notebook-list-old-format.list')
+
+class TestNotebookInfoListBackwardCompatibility(tests.TestCase):
+
+	def runTest(self):
+		# Check backward compatibility for old file format
+		# Format is name, value pair separated by whitespace (tab or space)
+		folder = self.setUpFolder()
+		file = folder.file('notebook-list-old-format.list')
+		lines = [
+			"_default_\tdebug\n",
+			"Notes\t~/Notes\n",
+			"   \n",
+			"# some comment \n",
+			"debug\t%s\n" % os_native_path('/home/user/code/zim.debug').replace('\\', '/'),
+			"Foo\\ Bar %s\n" % os_native_path('/home/user/Foo Bar').replace('\\', '/').replace(' ', '\\ '),
+		]
+		file.writelines(lines)
+
 		list = NotebookInfoList(file)
 		self.assertEqual(list[:], [
 			NotebookInfo(LocalFolder(path).uri) for path in
@@ -155,14 +171,18 @@ class TestResolveNotebook(tests.TestCase):
 
 	def runTest(self):
 		# First test some paths
-		for input, uri in (
-			('file:///foo/bar', 'file:///foo/bar'),
-			('~/bar', LocalFolder('~/bar').uri),
-		):
-			if os.name == 'nt':
-				input = input.replace('///', '///C:/')
-				if not '///C:/' in uri:
-					uri = uri.replace('///', '///C:/')
+		if os.name == 'nt':
+			test_paths = (
+				('file:///C:/foo/bar', 'file:///C:/foo/bar'),
+				('~/bar', LocalFolder('~/bar').uri),
+			)
+		else:
+			test_paths =(
+				('file:///foo/bar', 'file:///foo/bar'),
+				('~/bar', LocalFolder('~/bar').uri),
+			)
+
+		for input, uri in test_paths:
 			info = resolve_notebook(input)
 			self.assertEqual(info.uri, uri)
 
@@ -198,8 +218,16 @@ class TestBuildNotebook(tests.TestCase):
 import os
 import sys
 notebook = sys.argv[1]
-os.mkdir(notebook)
-os.mkdir(notebook + '/foo')
+notebookfile = notebook + "/notebook.zim"
+
+assert not os.path.exists(notebookfile), "Already exists: %s" % notebookfile
+
+try:
+	os.mkdir(notebook)
+	os.mkdir(notebook + '/foo')
+except FileExistsError:
+	pass
+
 for path in (
 	notebook + "/notebook.zim",
 	notebook + "/foo/bar.txt"
@@ -225,16 +253,21 @@ mount=%s %s
 			return dir
 
 		nbid = None
-		for uri, path in (
+		for uri, href in (
 			(self.notebookdir.uri, None), # first run triggers automount
 			(self.notebookdir.uri, None), # repeat to check automount & check uniqueness
 			(self.notebookdir.file('notebook.zim').uri, None),
-			(self.notebookdir.file('foo/bar.txt').uri, Path('foo:bar')),
+			(self.notebookdir.file('foo/bar.txt').uri, HRef.new_from_wiki_link('foo:bar')),
 		):
 			info = NotebookInfo(uri)
-			nb, p = build_notebook(info)
+			nb, pl = build_notebook(info)
 			self.assertEqual(nb.folder.path, self.notebookdir.path)
-			self.assertEqual(p, path)
+
+			if pl:
+				self.assertEqual(pl, href)
+			else:
+				self.assertIsNone(pl)
+
 			if nbid is None:
 				nbid = id(nb)
 			else:
@@ -450,24 +483,33 @@ class TestNotebook(tests.TestCase):
 		for link, wanted, cleaned in (
 			('~/test.txt', LocalFile('~/test.txt'), '~/test.txt'),
 			(r'~\test.txt', LocalFile('~/test.txt'), '~/test.txt'),
-			('file:///test.txt', LocalFile(os_native_path('file:///test.txt')), None),
-			('file:/test.txt', LocalFile(os_native_path('file:///test.txt')), None),
-			('file://localhost/test.txt', LocalFile(os_native_path('file:///test.txt')), None),
+			('~/test/', LocalFolder('~/test'), '~/test/'),
+			(os_native_path('file:///test.txt'), LocalFile(os_native_path('file:///test.txt')), None),
+			(os_native_path('file:/test.txt'), LocalFile(os_native_path('file:///test.txt')), None),
+			(os_native_path('file://localhost/test.txt'), LocalFile(os_native_path('file:///test.txt')), None),
+			('file:///C:/test.txt', LocalFile('file:///C:/test.txt'), None),
+			('file:///C:/test/', LocalFolder('file:///C:/test'), None),
 			('/test.txt', doc_root.file('test.txt'), '/test.txt'),
 			('../../notebook_document_root/test.txt', doc_root.file('test.txt'), '/test.txt'),
 			('./test.txt', dir.file('Foo/Bar/test.txt'), './test.txt'),
+			('./test/', dir.folder('Foo/Bar/test'), './test/'),
 			(r'.\test.txt', dir.file('Foo/Bar/test.txt'), './test.txt'),
 			('../test.txt', dir.file('Foo/test.txt'), '../test.txt'),
+			('../test/', dir.folder('Foo/test'), '../test/'),
 			(r'..\test.txt', dir.file('Foo/test.txt'), '../test.txt'),
 			('../Bar/Baz/test.txt', dir.file('Foo/Bar/Baz/test.txt'), './Baz/test.txt'),
+			('../Other/Baz/test.txt', dir.file('Foo/Other/Baz/test.txt'), '../Other/Baz/test.txt'),
+			('./../Other/Baz/test.txt', dir.file('Foo/Other/Baz/test.txt'), '../Other/Baz/test.txt'),
 			(r'C:\foo\bar', LocalFile('file:///C:/foo/bar'), None),
 			(r'Z:\foo\bar', LocalFile('file:///Z:/foo/bar'), None),
+			(r'Z:\foo\bar\\', LocalFolder('file:///Z:/foo/bar'), None),
 		):
-			#~ print link, '>>', self.notebook.resolve_file(link, path)
+			#print("== LINK", link)
+			#print('>>', self.notebook.resolve_file(link, path))
 			if cleaned is not None and not cleaned.startswith('/'):
 				cleaned = os_native_path(cleaned)
 			self.assertEqual(
-				adapt_from_oldfs(self.notebook.resolve_file(link, path)), wanted)
+				self.notebook.resolve_file(link, path), wanted)
 			self.assertEqual(
 				self.notebook.relative_filepath(wanted, path), cleaned)
 
@@ -941,10 +983,10 @@ class TestUpdateLinksOnMovePage(tests.TestCase):
 	def testMultipleLinksOnePage(self):
 		self.movePage(
 			pre=({
-					'A': 'test 123',
-					'A:A1': 'test 123',
-					'B': '[[A]]\n[[:A]]\n[[D]]\n[[A:A1]]',
-					'D': 'test 123',
+					'A': 'test 123\n',
+					'A:A1': 'test 123\n',
+					'B': '[[A]]\n[[:A]]\n[[D]]\n[[A:A1]]\n',
+					'D': 'test 123\n',
 				},
 				[('B', 'A'), ('B', 'A:A1'), ('B', 'D')]
 			),
@@ -1233,22 +1275,6 @@ class TestPage(TestPath):
 		page = self.generator('Foo')
 		page.set_parsetree(tree)
 
-		links = list(page.get_links())
-		self.assertEqual(links, [
-			('page', 'foo:bar', {}),
-			('page', 'bar', {}),
-		])
-
-		tags = list(page.get_tags())
-		self.assertEqual(tags, [
-			('baz', {'name': 'baz'}),
-		])
-
-		anchors = list(page.get_anchors())
-		self.assertEqual(anchors, [
-			('#bottom', {'name': 'bottom'}),
-		])
-
 		self.assertEqual(page.get_parsetree().tostring(), tree.tostring())
 			# ensure we didn't change the tree
 
@@ -1353,7 +1379,7 @@ class TestPage(TestPath):
 		page.set_parsetree(tree)
 		self.assertEqual(page.dump('wiki'), ['ABC\n'])
 		self.assertFalse(page.check_source_changed())
-		file.write('DEF')
+		file.write('DEF\n')
 		self.assertEqual(page.dump('wiki'), ['ABC\n'])
 		self.assertTrue(page.check_source_changed())
 		self.assertEqual(page.dump('wiki'), ['DEF\n'])
@@ -1598,9 +1624,47 @@ class TestBackgroundSave(tests.TestCase):
 		self.assertEqual(signals['stored-page'], [(page,)]) # post handler happened as well
 
 
-class AttachmentsFolderIsinstance(tests.TestCase):
+class TestFilesLayout(tests.TestCase):
 
-	def runTest(self):
+	def _test_page_vs_not_a_page(self, folder, layout, pagefile, notapagefile):
+		self.assertTrue(layout.is_source_file(pagefile))
+		self.assertFalse(layout.is_source_file(notapagefile))
+
+		self.assertEqual(layout.map_file(pagefile), (Path('Page'), FILE_TYPE_PAGE_SOURCE))
+		self.assertEqual(layout.map_file(notapagefile), (Path(':'), FILE_TYPE_ATTACHMENT))
+		self.assertEqual(layout.map_filepath(pagefile.relpath(folder)), (Path('Page'), FILE_TYPE_PAGE_SOURCE))
+		self.assertEqual(layout.map_filepath(notapagefile.relpath(folder)), (Path(':'), FILE_TYPE_ATTACHMENT))
+
+		self.assertEqual(layout.index_list_children(Path(':')), [Path('Page')])
+
+	def testCheckFirstLineForTextFiles(self):
+		folder = self.setUpFolder()
+		pagefile = folder.file('Page.txt')
+		pagefile.write('Content-Type: text/x-zim-wiki\n\nFoo Bar\n')
+		notapagefile = folder.file('NotAPage.txt')
+		notapagefile.write('Foo Bar\n')
+		layout = FilesLayout(folder, default_extension='.txt')
+		self._test_page_vs_not_a_page(folder, layout, pagefile, notapagefile)
+
+	def testNoCheckFirstLineForNonTextFiles(self):
+		folder = self.setUpFolder()
+		pagefile = folder.file('Page.md')
+		pagefile.write('Foo Bar\n')
+		notapagefile = folder.file('NotAPage.txt')
+		notapagefile.write('Foo Bar\n')
+		layout = FilesLayout(folder, default_extension='.md')
+		self._test_page_vs_not_a_page(folder, layout, pagefile, notapagefile)
+
+	def testInValidFileNamesRejected(self):
+		folder = self.setUpFolder()
+		pagefile = folder.file('Page.txt')
+		pagefile.write('Content-Type: text/x-zim-wiki\n\nFoo Bar\n')
+		notapagefile = folder.file('Not A Page.txt')
+		notapagefile.write('Content-Type: text/x-zim-wiki\n\nFoo Bar\n')
+		layout = FilesLayout(folder, default_extension='.txt')
+		self._test_page_vs_not_a_page(folder, layout, pagefile, notapagefile)
+
+	def testAttachmentsFolderIsinstance(self):
 		folder = self.setUpFolder()
 		layout = FilesLayout(folder)
 		afolder = layout.get_attachments_folder(Path('Test'))
