@@ -5,7 +5,7 @@
 
 import tests
 
-from zim.formats import ParseTree, StubLinker, OldParseTreeBuilder
+from zim.formats import ParseTree, StubLinker, BackwardParseTreeBuilderWithCleanup
 from zim.formats.wiki import Parser as WikiParser
 from zim.formats.wiki import Dumper as WikiDumper
 from zim.formats.html import Dumper as HtmlDumper
@@ -81,7 +81,7 @@ class TestWikiSyntaxNoPlugin(tests.TestCase):
 
 	def parseAndDump(self, text):
 		tree = WikiParser().parse(text)
-		self.assertEquals(list(tree.iter_tokens()), TABLE_TOKENS)
+		self.assertEqual(list(tree.iter_tokens()), TABLE_TOKENS)
 
 	def testWikiText(self):
 		self.parseAndDump(TABLE_WIKI_TEXT)
@@ -111,28 +111,28 @@ class TestTableObjectType(tests.TestCase):
 		self.assertIsNotNone(element)
 		model = self.otype.model_from_element(element.attrib, element)
 
-		builder = OldParseTreeBuilder() # XXX
+		builder = BackwardParseTreeBuilderWithCleanup() # XXX
 		builder.start('zim-tree')
 		self.otype.dump(builder, model)
 		builder.end('zim-tree')
 		tree = ParseTree(builder.close())
 
-		#self.assertEquals(list(tree.iter_tokens()), TABLE_TOKENS) -- XXX should work but doesn;t :(
-		self.assertEquals(''.join(WikiDumper().dump(tree)), TABLE_WIKI_TEXT[1:-1])
+		#self.assertEqual(list(tree.iter_tokens()), TABLE_TOKENS) -- XXX should work but doesn;t :(
+		self.assertEqual(''.join(WikiDumper().dump(tree)), TABLE_WIKI_TEXT[1:-1])
 
 	def testModelFromData(self):
 		notebook = self.setUpNotebook()
 		page = notebook.get_page(Path('Test'))
 		model = self.otype.model_from_data(notebook, page, {}, TABLE_WIKI_TEXT)
 
-		builder = OldParseTreeBuilder() # XXX
+		builder = BackwardParseTreeBuilderWithCleanup() # XXX
 		builder.start('zim-tree')
 		self.otype.dump(builder, model)
 		builder.end('zim-tree')
 		tree = ParseTree(builder.close())
 
-		#self.assertEquals(list(tree.iter_tokens()), TABLE_TOKENS) -- XXX should work but doesn;t :(
-		self.assertEquals(''.join(WikiDumper().dump(tree)), TABLE_WIKI_TEXT[1:-1])
+		#self.assertEqual(list(tree.iter_tokens()), TABLE_TOKENS) -- XXX should work but doesn;t :(
+		self.assertEqual(''.join(WikiDumper().dump(tree)), TABLE_WIKI_TEXT[1:-1])
 
 
 class TestPageViewNoPlugin(tests.TestCase):
@@ -147,7 +147,7 @@ class TestPageViewNoPlugin(tests.TestCase):
 		)
 		pageview.textview.get_buffer().set_modified(True) # Force re-interpretation of the buffer
 		tree = pageview.page.get_parsetree()
-		self.assertEquals(list(tree.iter_tokens()), TABLE_TOKENS)
+		self.assertEqual(list(tree.iter_tokens()), TABLE_TOKENS)
 
 
 class TestPageViewWithPlugin(TestPageViewNoPlugin):
@@ -174,7 +174,7 @@ class TestPageViewWithPlugin(TestPageViewNoPlugin):
 		# test modification ends up in page
 		tree = pageview.page.get_parsetree()
 		#print(tree.tostring())
-		elt = tree.find('table')
+		elt = tree.find_element('table')
 		self.assertIsNotNone(elt)
 		# FIXME: test content
 
@@ -190,7 +190,7 @@ class TestPageViewWithPlugin(TestPageViewNoPlugin):
 			action.activate()
 
 		tree = window.pageview.page.get_parsetree()
-		elt = tree.find('table')
+		elt = tree.find_element('table')
 		self.assertIsNotNone(elt)
 
 	def testInsertDialogCancelled(self):
@@ -205,7 +205,7 @@ class TestPageViewWithPlugin(TestPageViewNoPlugin):
 			action.activate()
 
 		tree = window.pageview.page.get_parsetree()
-		elt = tree.find('table')
+		elt = tree.find_element('table')
 		self.assertIsNone(elt)
 
 
