@@ -9,17 +9,19 @@ applications defined in desktop entry files.
 
 import sys
 import os
+import re
 import logging
 import subprocess
 import locale
 
 from gi.repository import GObject
+from gi.repository import GLib
 
-import zim.fs
 import zim.errors
 
-from zim.fs import File, SEP
-from zim.parsing import split_quoted_strings, is_uri_re, is_win32_path_re
+from zim.fs import adapt_from_oldfs
+from zim.newfs import SEP, is_abs_filepath, LocalFile, LocalFolder
+from zim.parsing import is_uri_re, is_win32_path_re
 
 
 logger = logging.getLogger('zim.applications')
@@ -69,8 +71,71 @@ def _split_environ_list(value):
 		raise ValueError
 
 
+
+_word_re = re.compile(r'''
+	(
+		"(\\"|[^"])*" |  # double quoted word
+		[^\s"]+          # word without spaces
+	)''', re.X)
+
+
+def split_quoted_strings(string):
+	'''Split a word list respecting quotes according to XDG desktop entry spec
+
+	Only supports double quotes as specified in the spec
+
+	This function always expect full words to be quoted, even if quotes
+	appear in the middle of a word, they are considered word
+	boundries.
+
+	( XDG Desktop Entry spec says full words must be quoted and
+	quotes in a word escaped, but doesn't specify what to do with
+	loose quotes in a string. Also this spec does not allow single
+	quote quotes)
+	'''
+	string = string.strip()
+	words = []
+	m = _word_re.match(string)
+	while m:
+		w = m.group(0)
+
+		words.append(w)
+		i = m.end()
+		string = string[i:].lstrip()
+		m = _word_re.match(string)
+
+	if string:
+		words += string.split() # unmatched quote ?
+
+	return [_unescape_quoted_string(w) for w in words if w]
+
+
+def _unescape_quoted_string(string):
+	# XDG Desktop entry spec says:"If an argument contains a reserved character
+	# the argument *must* be quoted."
+	# Therefore, unquoted arguments with backslash are invalid. However on
+	# Windows we may have created these, so for backward compatibility pass
+	# them through without unescaping.
+	# Unescaping here does not target \n etc. but \" and other reserved characters
+	# avoid touching alphabetic chars in case there are invalid paths in the string
+	if string[0] == '"' and string[-1] == '"':
+		return re.sub(r'\\(\W)', '\\1', string[1:-1])
+	else:
+		return string
+
+
+class ApplicationLookUpError(zim.errors.Error):
+	'''Error raised when an application is not found'''
+
+	description = None
+
+	def __init__(self, cmd):
+		self.msg = _('Cound not find application: %s') % cmd
+			# T: Error message when external application could not be found, %s is the command
+
+
 class ApplicationError(zim.errors.Error):
-	'''Error raises for error in sub process errors'''
+	'''Error raised for errors in the sub process'''
 
 	description = None
 
@@ -144,7 +209,7 @@ class Application(object):
 	@staticmethod
 	def _lookup(cmd):
 		'''Lookup cmd in PATH'''
-		if zim.fs.isabs(cmd):
+		if is_abs_filepath(cmd):
 			if os.path.isfile(cmd):
 				return cmd
 			else:
@@ -197,7 +262,7 @@ class Application(object):
 
 		# Expand home dir
 		if argv[0].startswith('~'):
-			cmd = File(argv[0]).path
+			cmd = LocalFile(argv[0]).path
 			argv = list(argv)
 			argv[0] = cmd
 
@@ -350,13 +415,13 @@ class Application(object):
 			try:
 				pid, stdin, stdout, stderr = \
 					GObject.spawn_async(argv, flags=flags, working_directory=cwd)
-			except GObject.GError:
+			except (GObject.GError, GLib.Error):
 				if _CAN_CALL_FLATPAK_HOST_COMMAND:
 					pid, stdin, stdout, stderr = \
 						GObject.spawn_async(_FLATPAK_HOSTCOMMAND_PREFIX + argv, flags=flags, working_directory=cwd)
 				else:
 					raise
-		except GObject.GError:
+		except (GObject.GError, GLib.Error):
 			from zim.gui.widgets import ErrorDialog
 			ErrorDialog(None, _('Failed running: %s') % argv[0]).run()
 				#~ # T: error when application failed to start
@@ -407,7 +472,7 @@ class WebBrowser(Application):
 			raise NotImplementedError('WebBrowser can not handle callback')
 
 		for url in args:
-			if isinstance(url, (zim.fs.File, zim.fs.Dir)):
+			if hasattr(url, 'uri'):
 				url = url.uri
 			logger.info('Opening in webbrowser: %s', url)
 
@@ -445,7 +510,8 @@ class StartFile(Application):
 			raise NotImplementedError('os.startfile does not support a callback')
 
 		for arg in args:
-			if isinstance(arg, (zim.fs.File, zim.fs.Dir)):
+			arg = adapt_from_oldfs(arg)
+			if isinstance(arg, (LocalFile, LocalFolder)):
 				path = os.path.normpath(arg.path).replace('/', SEP) # msys can use '/' instead of '\\'
 			elif is_uri_re.match(arg) and not is_win32_path_re.match(arg):
 				# URL or e.g. mailto: or outlook: URI
