@@ -1,5 +1,5 @@
 
-# Copyright 2008-2018 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2008-2022 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 import os
 import sys
@@ -13,14 +13,15 @@ logger = logging.getLogger('zim.gui')
 
 
 from zim.config import data_file, value_is_coord, ConfigDict, Boolean, ConfigManager
-from zim.signals import DelayedCallback
+from zim.signals import DelayedCallback, ConnectorMixin
 
 from zim.notebook import Path, Page, LINK_DIR_BACKWARD, PageNotAvailableError
 from zim.notebook.index import IndexNotFoundError
 from zim.notebook.operations import ongoing_operation
 from zim.history import History, HistoryPath
 
-from zim.actions import action, toggle_action, radio_action, radio_option, get_gtk_actiongroup, \
+from zim.actions import action, toggle_action, radio_action, radio_option, \
+	get_gtk_actiongroup, initialize_actiongroup, \
 	PRIMARY_MODIFIER_STRING, PRIMARY_MODIFIER_MASK
 from zim.gui.widgets import \
 	MenuButton, \
@@ -28,18 +29,20 @@ from zim.gui.widgets import \
 	ErrorDialog, FileDialog, ProgressDialog, MessageDialog, QuestionDialog, \
 	ScrolledTextView, \
 	gtk_popup_at_pointer, \
-	TOP
+	TOP, BOTTOM
 
 from zim.gui.navigation import NavigationModel
 from zim.gui.uiactions import UIActions
-from zim.gui.customtools import CustomToolManagerUI
+from zim.gui.customtools import CustomToolManager, CustomToolManagerUI
 from zim.gui.insertedobjects import InsertedObjectUI
 
 from zim.gui.pageview import PageView
+from zim.gui.pageview.editbar import ToolBarEditBarManager
 from zim.gui.notebookview import NotebookView
 
 from zim.plugins import ExtensionBase, extendable, PluginManager
-from zim.gui.actionextension import ActionExtensionBase
+from zim.gui.actionextension import ActionExtensionBase, \
+	populate_toolbar_with_actions, os_default_headerbar
 
 
 MENU_ACTIONS = (
@@ -62,6 +65,8 @@ if sys.platform == "darwin":
 #: Preferences for the user interface
 ui_preferences = (
 	# key, type, category, label, default
+	('show_headerbar', 'bool', 'Interface', _('Show controls in the window decoration') + '\n' + _('This option requires restart of the application'), os_default_headerbar),
+		# T: option for plugin preferences
 	('toggle_on_ctrlspace', 'bool', 'Interface', _('Use %s to switch to the side pane') % (PRIMARY_MODIFIER_STRING + '<Space>'), False),
 		# T: Option in the preferences dialog - %s will map to either <Control><Space> or <Command><Space> key binding
 		# default value is False because this is mapped to switch between
@@ -124,7 +129,7 @@ class MainWindowExtension(ActionExtensionBase):
 		self.destroy()
 
 
-class WindowBaseMixin(object):
+class WindowBaseMixin(ConnectorMixin, object):
 	'''Common logic between MainWindow and PageWindow'''
 
 	def _init_fullscreen_headerbar(self):
@@ -151,6 +156,8 @@ class WindowBaseMixin(object):
 		self._fullscreen_eventbox.connect('leave-notify-event', self._on_fullscreen_eventbox_leave)
 
 	def _on_fullscreen_eventbox_enter(self, *a):
+		if self._headerbar is None and self._toolbar and self._toolbar.get_visible():
+			return # Do not show fullscreen headerbar if headerbar controls are in toolbar *and* visible
 		self._in_fullscreen_eventbox = True
 		self._update_fullscreen_revealer()
 
@@ -167,15 +174,91 @@ class WindowBaseMixin(object):
 
 	def _populate_headerbars(self):
 		for headerbar in (self._headerbar, self._fullscreen_headerbar):
-			self._populate_headerbar(headerbar)
-			headerbar.show_all()
+			if headerbar is not None:
+				self._populate_headerbar(headerbar)
+				headerbar.show_all()
 
 		for c in self._fullscreen_headerbar.get_children():
 			if isinstance(c, Gtk.MenuButton):
 				c.connect('toggled', self._update_fullscreen_revealer)
 
+	def _init_toolbar(self):
+		# One time setup of the toolbar widget
+		self._toolbar = Gtk.Toolbar()
+		self._toolbar_editbar_manager = ToolBarEditBarManager(self.pageview, self._toolbar)
+		self._show_edit_bar_controls_in_toolbar = self.pageview.preferences['show_edit_bar']
+
+		self.setup_toolbar()
+
+		def on_extensions_changed(o, obj):
+			if obj in (self, self.pageview):
+				self.update_toolbar()
+
+		self.connectto(PluginManager, 'extensions-changed', on_extensions_changed)
+
+		def on_changed_update(o, *a):
+			self.update_toolbar()
+
+		self.connectto(CustomToolManager(), 'changed', on_changed_update)
+		self.connectto(self.pageview.preferences, 'changed', on_changed_update)
+
+	def setup_toolbar(self, show=None, position=None, show_edit_bar_controls=None):
+		# Default setup for toolbar - can be called multiple times - also used by "toolbar" plugin
+		try:
+			self.remove(self._toolbar)
+		except ValueError:
+			pass
+
+		# Defaults
+		if show is None:
+			show = not self.preferences['show_headerbar']
+
+		position = TOP if position is None else position
+
+		if show_edit_bar_controls is None:
+			self._show_edit_bar_controls_in_toolbar = not self.pageview.preferences['show_edit_bar']
+			self.pageview.set_edit_bar_visible(None)
+		else:
+			self._show_edit_bar_controls_in_toolbar = show_edit_bar_controls
+			self.pageview.set_edit_bar_visible(not show_edit_bar_controls)
+
+		# Set toolbar in window
+		if show:
+			self.add_bar(self._toolbar, position=position)
+			if position in (TOP, BOTTOM):
+				self._toolbar.set_orientation(Gtk.Orientation.HORIZONTAL)
+			else: # LEFT, RIGHT
+				self._toolbar.set_orientation(Gtk.Orientation.VERTICAL)
+			self._toolbar.show()
+			self.update_toolbar()
+		else:
+			self._toolbar.hide()
+
+		return self._toolbar
+
+	def update_toolbar(self):
+		# This method updates the toolbar content. See setup_toolbar() to
+		# control placement and visibility.
+		if self._toolbar.get_visible():
+			for item in self._toolbar.get_children():
+				self._toolbar.remove(item)
+			self._populate_toolbar(self._toolbar)
+			self._toolbar.show_all()
+
+	def _populate_toolbar_inner(self, toolbar):
+		if self._show_edit_bar_controls_in_toolbar:
+			self._toolbar_editbar_manager.populate_toolbar(toolbar)
+
+		populate_toolbar_with_actions(
+			self._toolbar, self, self.pageview,
+			include_headercontrols=(not self.preferences['show_headerbar']),
+			include_customtools=True
+		)
+
 	def set_title(self, text):
-		self._headerbar.set_title(text)
+		Gtk.Window.set_title(self, text)
+		if self._headerbar is not None:
+			self._headerbar.set_title(text)
 		self._fullscreen_headerbar.set_title(text)
 
 	@toggle_action(_('_Fullscreen'), 'F11', icon='view-fullscreen-symbolic', init=False) # T: Menu item
@@ -210,23 +293,21 @@ class WindowBaseMixin(object):
 			pass
 		self.emit('readonly-changed', readonly)
 
-	def setup_toggle_editable_state(self, editable_uistate):
-		'''Setup sensitivity of the "toggle_editable" action
+	def set_toggle_editable_state(self, editable_uistate):
+		'''Set sensitivity of the "toggle_editable" action
 		@param editable_uistate: default state if control is sensitive
 		'''
 		if self.notebook.readonly or self.page.readonly:
-			if not self.toggle_editable.get_sensitive():
-				return
-
-			self.toggle_editable.set_sensitive(False)
-			self._set_tooltip_hack(_('Page is read-only and cannot be edited')) # T: message in toggle editable tooltip
-		else:
 			if self.toggle_editable.get_sensitive():
-				return
+				self.toggle_editable.set_sensitive(False)
+				self._set_tooltip_hack(_('Page is read-only and cannot be edited')) # T: message in toggle editable tooltip
+
+		else:
+			if not self.toggle_editable.get_sensitive():
+				self.toggle_editable.set_sensitive(True)
+				self._set_tooltip_hack(self.toggle_editable.tooltip) # reset to default
 
-			self.toggle_editable.set_sensitive(True)
 			self.toggle_editable(editable_uistate)
-			self._set_tooltip_hack(self.toggle_editable.tooltip) # reset to default
 
 	def _set_tooltip_hack(self, text):
 		for proxy in self.toggle_editable._proxies: # XXX
@@ -237,9 +318,9 @@ class WindowBaseMixin(object):
 		def _change_style_on_toggle(button):
 			context = button.get_style_context()
 			if button.get_active():
-				context.remove_class("suggested-action")
+				context.remove_class(Gtk.STYLE_CLASS_SUGGESTED_ACTION)
 			else:
-				context.add_class("suggested-action")
+				context.add_class(Gtk.STYLE_CLASS_SUGGESTED_ACTION)
 		_change_style_on_toggle(button)
 		button.connect('toggled', _change_style_on_toggle)
 
@@ -278,7 +359,11 @@ class WindowBaseMixin(object):
 			# Sidepane open and should close automatically
 			self.toggle_panes(False)
 
-	def toggle_sidepane_focus(self, *a):
+	@action(_('Focus Sidepane')) # T: menu item
+	def focus_sidepane_key_toggle(self):
+		self.do_focus_sidepane_key_toggle()
+
+	def do_focus_sidepane_key_toggle(self, *a):
 		'''Switch focus between the textview and the page index.
 		Automatically opens the sidepane if it is closed
 		(but sets a property to automatically close it again).
@@ -329,10 +414,7 @@ class MainWindow(WindowBaseMixin, Window):
 		self.hideonclose = False
 
 		self.preferences = ConfigManager.preferences['GtkInterface']
-		self.preferences.define(
-			toggle_on_ctrlspace=Boolean(False),
-			always_use_last_cursor_pos=Boolean(True),
-		)
+		self.preferences.define({p[0]: Boolean(p[-1]) for p in ui_preferences})
 		self.preferences.connect('changed', self.do_preferences_changed)
 
 		self.maximized = False
@@ -426,13 +508,18 @@ class MainWindow(WindowBaseMixin, Window):
 		fname = 'menubar.xml'
 		self.uimanager.add_ui_from_string(data_file(fname).read())
 
-		# header Bar
-		self._headerbar = Gtk.HeaderBar()
-		self._headerbar.set_show_close_button(True)
-		self.set_titlebar(self._headerbar)
+		# header & tool bars
+		if self.preferences['show_headerbar']:
+			self._headerbar = Gtk.HeaderBar()
+			self._headerbar.set_show_close_button(True)
+			self.set_titlebar(self._headerbar)
+		else:
+			self._headerbar = None
 
 		self._populate_headerbars()
 
+		self._init_toolbar()
+
 		# Do this last, else menu items show up in wrong place
 		self._customtools = CustomToolManagerUI(self.uimanager, self.pageview)
 		self._insertedobjects = InsertedObjectUI(self.uimanager, self.pageview)
@@ -470,6 +557,8 @@ class MainWindow(WindowBaseMixin, Window):
 			self.open_page_home()
 
 		PluginManager.register_new_extendable(self.pageview)
+		initialize_actiongroup(self, 'win')
+
 		self.pageview.grab_focus()
 
 	def _populate_headerbar(self, headerbar):
@@ -490,6 +579,34 @@ class MainWindow(WindowBaseMixin, Window):
 		self._style_toggle_editable_button(button)
 		headerbar.pack_end(button)
 
+	def _populate_toolbar(self, toolbar):
+		# Add default controls
+		if not self.preferences['show_headerbar']:
+			for action in (
+				self.open_page_back,
+				self.open_page_home,
+				self.open_page_forward,
+			):
+				toolbar.insert(action.create_tool_button(), -1)
+			toolbar.insert(Gtk.SeparatorToolItem(), -1)
+
+			item = self.toggle_editable.create_tool_button(connect_button=False)
+			item.set_action_name('win.toggle_editable')
+			self._style_toggle_editable_button(item)
+			toolbar.insert(item, -1)
+			toolbar.insert(Gtk.SeparatorToolItem(), -1)
+
+			self._populate_toolbar_inner(toolbar)
+
+			space = Gtk.SeparatorToolItem()
+			space.set_draw(False)
+			space.set_expand(True)
+			toolbar.insert(space, -1)
+
+			toolbar.insert(self._uiactions.show_search.create_tool_button(), -1)
+		else:
+			self._populate_toolbar_inner(toolbar)
+
 	@action(_('_Close'), '<Primary>W') # T: Menu item
 	def close(self):
 		'''Menu action for close. Will hide when L{hideonclose} is set,
@@ -549,14 +666,14 @@ class MainWindow(WindowBaseMixin, Window):
 			# see bug lp:620315)
 			group.connect( # <Alt><Space>
 				space, Gdk.ModifierType.MOD1_MASK, Gtk.AccelFlags.VISIBLE,
-				self.toggle_sidepane_focus)
+				self.do_focus_sidepane_key_toggle)
 
 		# Toggled by preference menu, also causes issues with international
 		# layouts - esp. when switching input method on Meta-Space
 		if self.preferences['toggle_on_ctrlspace']:
 			group.connect( # <Primary><Space>
 				space, PRIMARY_MODIFIER_MASK, Gtk.AccelFlags.VISIBLE,
-				self.toggle_sidepane_focus)
+				self.do_focus_sidepane_key_toggle)
 
 		self.add_accel_group(group)
 		self._switch_focus_accelgroup = group
@@ -622,6 +739,15 @@ class MainWindow(WindowBaseMixin, Window):
 		Gtk.AccelMap.get().connect('changed', on_accel_map_changed)
 
 	def save_uistate(self):
+		if not self.pageview._zim_extendable_registered:
+			return
+			# Not allowed to save before plugins are loaded, could overwrite
+			# pane state based on empty panes
+
+		cursor = self.pageview.get_cursor_pos()
+		scroll = self.pageview.get_scroll_pos()
+		self.history.set_state(self.page, cursor, scroll)
+
 		if self.is_visible() and not self.isfullscreen:
 			self.uistate['windowpos'] = tuple(self.get_position())
 			self.uistate['windowsize'] = tuple(self.get_size())
@@ -651,7 +777,7 @@ class MainWindow(WindowBaseMixin, Window):
 				logger.debug("Unused mouse button %i", event.button)
 		#~ return Window.do_button_press_event(self, event)
 
-	def open_page(self, path, anchor=None):
+	def open_page(self, path, anchor=None, anchor_fail_silent=False):
 		'''Method to open a page in the mainwindow, and menu action for
 		the "jump to" menu item.
 
@@ -681,7 +807,7 @@ class MainWindow(WindowBaseMixin, Window):
 
 		if self.page and id(self.page) == id(page):
 			if anchor:
-				self.pageview.navigate_to_anchor(anchor)
+				self.pageview.navigate_to_anchor(anchor, fail_silent=anchor_fail_silent)
 			return
 		elif self.page:
 			self.pageview.save_changes() # XXX - should connect to signal instead of call here
@@ -689,10 +815,6 @@ class MainWindow(WindowBaseMixin, Window):
 			if self.page.modified:
 				return False # Assume SavePageErrorDialog was shown and cancelled
 
-			old_cursor = self.pageview.get_cursor_pos()
-			old_scroll = self.pageview.get_scroll_pos()
-			self.history.set_state(self.page, old_cursor, old_scroll)
-
 			self.save_uistate()
 
 		logger.info('Open page: %s (%s)', page, path)
@@ -717,7 +839,7 @@ class MainWindow(WindowBaseMixin, Window):
 		self.pageview.set_page(page, cursor)
 
 		if anchor:
-			self.pageview.navigate_to_anchor(anchor)
+			self.pageview.navigate_to_anchor(anchor, fail_silent=anchor_fail_silent)
 
 		self.emit('page-changed', page)
 
@@ -727,7 +849,7 @@ class MainWindow(WindowBaseMixin, Window):
 		self.update_buttons_history()
 		self.update_buttons_hierarchy()
 		self._update_window_title()
-		self.setup_toggle_editable_state(not self.uistate['readonly'])
+		self.set_toggle_editable_state(not self.uistate['readonly'])
 
 	def _update_window_title(self):
 		if self.notebook.readonly or (self.page and self.page.readonly):
@@ -766,7 +888,7 @@ class MainWindow(WindowBaseMixin, Window):
 	@action(
 		_('_Back'), verb_icon='go-previous-symbolic', # T: Menu item
 		accelerator='<alt>Left', alt_accelerator='XF86Back',
-		tooltip=_('Go back')
+		tooltip=_('Go back') # T: tooltip for navigation button
 	)
 	def open_page_back(self):
 		'''Menu action to open the previous page from the history
@@ -779,7 +901,7 @@ class MainWindow(WindowBaseMixin, Window):
 	@action(
 		_('_Forward'), verb_icon='go-next-symbolic', # T: Menu item
 		accelerator='<alt>Right', alt_accelerator='XF86Forward',
-		tooltip=_('Go forward')
+		tooltip=_('Go forward') # T: tooltip for navigation button
 	)
 	def open_page_forward(self):
 		'''Menu action to open the next page from the history
@@ -883,16 +1005,24 @@ class PageWindow(WindowBaseMixin, Window):
 		'readonly-changed': (GObject.SignalFlags.RUN_LAST, None, (bool,)),
 	}
 
-	def __init__(self, notebook, page, anchor, navigation, editable=True):
+	def __init__(self, notebook, page, navigation, editable=True):
 		Window.__init__(self)
 		self._block_toggle_panes = False
 		self._sidepane_autoclose = False
 		self.navigation = navigation
 		self.notebook = notebook
 		self.page = notebook.get_page(page)
-		self._headerbar = Gtk.HeaderBar()
-		self._headerbar.set_show_close_button(True)
-		self.set_titlebar(self._headerbar)
+
+		self.preferences = ConfigManager.preferences['GtkInterface']
+		self.preferences.define({p[0]: Boolean(p[-1]) for p in ui_preferences})
+		#self.preferences.connect('changed', self.do_preferences_changed)
+
+		if self.preferences['show_headerbar']:
+			self._headerbar = Gtk.HeaderBar()
+			self._headerbar.set_show_close_button(True)
+			self.set_titlebar(self._headerbar)
+		else:
+			self._headerbar = None
 
 		self._init_fullscreen_headerbar()
 		self._populate_headerbars()
@@ -940,6 +1070,8 @@ class PageWindow(WindowBaseMixin, Window):
 		self.menubar = self.uimanager.get_widget('/menubar')
 		self.add_bar(self.menubar, position=TOP)
 
+		self._init_toolbar()
+
 		# Close window when page is moved or deleted
 		def on_notebook_change(o, path, *a):
 			if path == self.page or self.page.ischild(path):
@@ -951,7 +1083,7 @@ class PageWindow(WindowBaseMixin, Window):
 		notebook.connect('moved-page', on_notebook_change)
 
 		# setup state
-		self.setup_toggle_editable_state(editable)
+		self.set_toggle_editable_state(editable)
 
 		# on close window
 		def do_delete_event(*a):
@@ -961,8 +1093,8 @@ class PageWindow(WindowBaseMixin, Window):
 		self.connect('delete-event', do_delete_event)
 
 		PluginManager.register_new_extendable(self.pageview)
-		if anchor:
-			self.pageview.navigate_to_anchor(anchor)
+		initialize_actiongroup(self, 'win')
+
 		self.pageview.grab_focus()
 
 	def _populate_headerbar(self, headerbar):
@@ -973,7 +1105,34 @@ class PageWindow(WindowBaseMixin, Window):
 		self._style_toggle_editable_button(button)
 		headerbar.pack_end(button)
 
+	def _populate_toolbar(self, toolbar):
+		# Add default controls
+		if not self.preferences['show_headerbar']:
+			item = self.toggle_editable.create_tool_button(connect_button=False)
+			item.set_action_name('win.toggle_editable')
+			self._style_toggle_editable_button(item)
+			toolbar.insert(item, -1)
+			toolbar.insert(Gtk.SeparatorToolItem(), -1)
+
+			self._populate_toolbar_inner(toolbar)
+
+			space = Gtk.SeparatorToolItem()
+			space.set_draw(False)
+			space.set_expand(True)
+			toolbar.insert(space, -1)
+
+			# FIXME: should go in menu popover
+			item = self.toggle_fullscreen.create_tool_button(connect_button=False)
+			item.set_action_name('win.toggle_fullscreen')
+			toolbar.insert(item, -1)
+		else:
+			self._populate_toolbar_inner(toolbar)
+
 	def save_uistate(self):
+		if not self.pageview._zim_extendable_registered:
+			return
+			# Not allowed to save before plugins are loaded, could overwrite
+			# pane state based on empty panes
 		self.uistate['windowsize'] = tuple(self.get_size())
 		Window.save_uistate(self) # takes care of sidepane positions etc.
 
