@@ -19,7 +19,7 @@ import signal
 logger = logging.getLogger('zim')
 
 import zim
-import zim.fs
+import zim.newfs
 import zim.errors
 import zim.config
 import zim.config.basedirs
@@ -28,7 +28,7 @@ from zim import __version__
 
 from zim.utils import get_module, lookup_subclass
 from zim.errors import Error
-from zim.notebook import Notebook, Path, \
+from zim.notebook import Notebook, Path, HRef, \
 	get_notebook_list, resolve_notebook, build_notebook
 from zim.formats import get_format
 
@@ -44,14 +44,19 @@ class HelpCommand(Command):
 	'''Class implementing the C{--help} command'''
 
 	usagehelp = '''\
-usage: zim [OPTIONS] [NOTEBOOK [PAGE]]
+usage: zim [OPTIONS] [NOTEBOOK [PAGE_LINK]]
+   or: zim --gui [OPTIONS] [NOTEBOOK [PAGE_LINK]]
    or: zim --server [OPTIONS] [NOTEBOOK]
    or: zim --export [OPTIONS] NOTEBOOK [PAGE]
    or: zim --search NOTEBOOK QUERY
-   or: zim --index  NOTEBOOK
+   or: zim --index  [OPTIONS] NOTEBOOK
    or: zim --plugin PLUGIN [ARGUMENTS]
-   or: zim --manual [OPTIONS] [PAGE]
+   or: zim --manual [OPTIONS] [PAGE_LINK]
    or: zim --help
+
+NOTEBOOK can be a local file path, a local file URI or a notebook name
+PAGE is be a fully specified page name
+PAGE_LINK is a fully specified page name optionally extended with an anchor ID
 '''
 	optionhelp = '''\
 General Options:
@@ -76,13 +81,14 @@ GUI Options:
 
 Server Options:
   --port           port to use (defaults to 8080)
-  --template       name of the template to use
+  --template       name or filepath of the template to use
+  --private        serve only to localhost
   --gui            run the gui wrapper for the server
 
 Export Options:
   -o, --output     output directory (mandatory option)
   --format         format to use (defaults to 'html')
-  --template       name of the template to use
+  --template       name or filepath of the template to use
   --root-url       url to use for the document root
   --index-page     index page name
   -r, --recursive  when exporting a page, also export sub-pages
@@ -93,7 +99,7 @@ Search Options:
   None
 
 Index Options:
-  None
+  -f, --flush      flush the index first and force re-building
 
 Try 'zim --manual' for more help.
 '''
@@ -137,7 +143,7 @@ class NotebookCommand(Command):
 	def get_notebook_argument(self):
 		'''Get the notebook and page arguments for this command
 		@returns: a 2-tuple of an L{NotebookInfo} object and an
-		optional L{Path} or C{(None, None)} if the notebook
+		optional L{HRef} or C{(None, None)} if the notebook
 		argument is optional and not given
 		@raises NotebookLookupError: if the notebook is mandatory and
 		not given, or if it is given but could not be resolved
@@ -159,10 +165,9 @@ class NotebookCommand(Command):
 				# T: error message
 
 		if len(self.arguments) > 1 \
-		and self.arguments[1] in ('PAGE', '[PAGE]') \
+		and self.arguments[1] in ('PAGE', '[PAGE]', 'PAGE_LINK', '[PAGE_LINK]') \
 		and args[1] is not None:
-			pagename = Path.makeValidPageName(args[1])
-			return notebookinfo, Path(pagename)
+			return notebookinfo, HRef.new_from_wiki_link(args[1])
 		else:
 			return notebookinfo, None
 
@@ -172,7 +177,7 @@ class NotebookCommand(Command):
 		@param ensure_uptodate: if C{True} index is updated when needed.
 		Only set to C{False} when index update is handled explicitly
 		(e.g. in the main gui).
-		@returns: a L{Notebook} object and a L{Path} object or C{None}
+		@returns: a L{Notebook} object and a C{HRef} object or C{None}
 		@raises NotebookLookupError: if the notebook could not be
 		resolved or is not given
 		@raises FileNotFoundError: if the notebook location does not
@@ -180,23 +185,23 @@ class NotebookCommand(Command):
 		'''
 		# Explicit page argument has priority over implicit from uri
 		# mounting is attempted by zim.notebook.build_notebook()
-		notebookinfo, page = self.get_notebook_argument() 	# can raise NotebookLookupError
+		notebookinfo, pagelink = self.get_notebook_argument() 	# can raise NotebookLookupError
 		if not notebookinfo:
 			raise NotebookLookupError(_('Please specify a notebook'))
-		notebook, uripage = build_notebook(notebookinfo) # can raise FileNotFound
+		notebook, uripagelink = build_notebook(notebookinfo) # can raise FileNotFoundError
 
 		if ensure_uptodate and not notebook.index.is_uptodate:
 			for info in notebook.index.update_iter():
 				#logger.info('Indexing %s', info)
 				pass # TODO meaningful info for above message
 
-		return notebook, page or uripage
+		return notebook, pagelink or uripagelink
 
 
 class GuiCommand(NotebookCommand, GtkCommand):
 	'''Class implementing the C{--gui} command and run the gtk interface'''
 
-	arguments = ('[NOTEBOOK]', '[PAGE]')
+	arguments = ('[NOTEBOOK]', '[PAGE_LINK]')
 	options = (
 		('list', '', 'show the list with notebooks instead of\nopening the default notebook'),
 		('geometry=', '', 'window size and position as WxH+X+Y'),
@@ -223,7 +228,7 @@ class GuiCommand(NotebookCommand, GtkCommand):
 		if self.opts.get('list'):
 			notebookinfo = prompt_notebook_list()
 		else:
-			notebookinfo, page = self.get_notebook_argument()
+			notebookinfo, pagelink = self.get_notebook_argument()
 
 			if notebookinfo is None:
 				notebookinfo = self.get_default_or_only_notebook()
@@ -236,14 +241,14 @@ class GuiCommand(NotebookCommand, GtkCommand):
 			return None, None # Cancelled prompt
 
 		try:
-			notebook, uripage = build_notebook(notebookinfo) # can raise FileNotFound
+			notebook, uripagelink = build_notebook(notebookinfo) # can raise FileNotFound
 		except FileNotFoundError:
 			if used_default:
 				# Default notebook went missing? Fallback to dialog to allow changing it
 				notebookinfo = prompt_notebook_list()
 				if notebookinfo is None:
 					return None, None # Cancelled prompt
-				notebook, uripage = build_notebook(notebookinfo) # can raise FileNotFound
+				notebook, uripagelink = build_notebook(notebookinfo) # can raise FileNotFound
 			else:
 				raise
 
@@ -252,7 +257,7 @@ class GuiCommand(NotebookCommand, GtkCommand):
 				#logger.info('Indexing %s', info)
 				pass # TODO meaningful info for above message
 
-		return notebook, page or uripage
+		return notebook, pagelink or uripagelink
 
 	def run(self):
 		from gi.repository import Gtk
@@ -264,23 +269,23 @@ class GuiCommand(NotebookCommand, GtkCommand):
 				if isinstance(w, MainWindow)
 		]
 
-		notebook, page = self.build_notebook()
+		notebook, pagelink = self.build_notebook()
 		if notebook is None:
 			logger.debug('NotebookDialog cancelled - exit')
 			return
 
 		for window in windows:
 			if window.notebook.uri == notebook.uri:
-				self._present_window(window, page)
+				self._present_window(window, pagelink)
 				return window
 		else:
-			return self._run_new_window(notebook, page)
+			return self._run_new_window(notebook, pagelink)
 
-	def _present_window(self, window, page):
+	def _present_window(self, window, pagelink):
 		window.present()
 
-		if page:
-			window.open_page(page)
+		if pagelink:
+			window.open_page(Path(pagelink.names), anchor=pagelink.anchor)
 
 		geometry = self.opts.get('geometry', None)
 		if geometry is not None:
@@ -289,7 +294,7 @@ class GuiCommand(NotebookCommand, GtkCommand):
 		if self.opts.get('fullscreen', False):
 			window.toggle_fullscreen(True)
 
-	def _run_new_window(self, notebook, page):
+	def _run_new_window(self, notebook, pagelink):
 		from gi.repository import GObject
 
 		from zim.gui.mainwindow import MainWindow
@@ -302,9 +307,9 @@ class GuiCommand(NotebookCommand, GtkCommand):
 			if not preferences['plugins']:
 				pluginmanager.load_plugins_from_preferences(
 					[ # Default plugins
-						'pageindex', 'pathbar',
+						'pageindex', 'pathbar', 'toolbar',
 						'insertsymbol', 'printtobrowser',
-						'versioncontrol',
+						'versioncontrol', 'osx_menubar'
 					]
 				)
 			else:
@@ -318,15 +323,18 @@ class GuiCommand(NotebookCommand, GtkCommand):
 
 			preferences['plugins_list_version'] = '0.70'
 
+		page = Path(pagelink.names) if pagelink else None
 		window = MainWindow(
 			notebook,
 			page=page,
 			**self.get_options('geometry', 'fullscreen')
 		)
 		window.present()
+		if pagelink and pagelink.anchor:
+			window.open_page(Path(pagelink.names), anchor=pagelink.anchor)
 
 		if not window.notebook.index.is_uptodate:
-			window._uiactions.reload_index(update_only=True) # XXX
+			window._uiactions.check_and_update_index(update_only=True) # XXX
 		else:
 			# Start a lightweight background check of the index
 			# put a small delay to ensure window is shown before we start
@@ -341,13 +349,13 @@ class GuiCommand(NotebookCommand, GtkCommand):
 class ManualCommand(GuiCommand):
 	'''Like L{GuiCommand} but always opens the manual'''
 
-	arguments = ('[PAGE]',)
+	arguments = ('[PAGE_LINK]',)
 	options = tuple(t for t in GuiCommand.options if t[0] != 'list')
 		# exclude --list
 
 	def run(self):
 		from zim.config import data_dir
-		self.arguments = ('NOTEBOOK', '[PAGE]') # HACK
+		self.arguments = ('NOTEBOOK', '[PAGE_LINK]') # HACK
 		self.args.insert(0, data_dir('manual').path)
 		return GuiCommand.run(self)
 
@@ -362,15 +370,19 @@ class ServerCommand(NotebookCommand):
 		('port=', 'p', 'port number to use (defaults to 8080)'),
 		('template=', 't', 'name or path of the template to use'),
 		('standalone', '', 'start a single instance, no background process'),
+		('private', '', 'serve only to localhost')
 	)
 
 	def run(self):
 		import zim.www
-		self.opts['port'] = int(self.opts.get('port', 8080))
-		self.opts.setdefault('template', 'Default')
-		notebook, page = self.build_notebook()
+		from zim.templates import get_template
 
-		self.server = httpd = zim.www.make_server(notebook, public=True, **self.get_options('template', 'port'))
+		port = int(self.opts.get('port', 8080))
+		template = get_template('html', self.opts.get('template', 'Default'), pwd=self.pwd)
+		notebook, x = self.build_notebook()
+		is_public = not self.opts.get('private', False)
+
+		self.server = httpd = zim.www.make_server(notebook, public=is_public, template=template, port=port)
 			# server attribute used in testing to stop sever in thread
 		logger.info("Serving HTTP on %s port %i...", httpd.server_name, httpd.server_port)
 		httpd.serve_forever()
@@ -390,8 +402,11 @@ class ServerGuiCommand(NotebookCommand, GtkCommand):
 
 	def run(self):
 		import zim.gui.server
-		self.opts['port'] = int(self.opts.get('port', 8080))
-		notebookinfo, page = self.get_notebook_argument()
+		from zim.templates import get_template
+
+		port = int(self.opts.get('port', 8080))
+		template = get_template('html', self.opts.get('template', 'Default'), pwd=self.pwd)
+		notebookinfo, x = self.get_notebook_argument()
 		if notebookinfo is None:
 			# Prefer default to be selected in drop down, user can still change
 			notebookinfo = self.get_default_or_only_notebook()
@@ -399,7 +414,8 @@ class ServerGuiCommand(NotebookCommand, GtkCommand):
 		window = zim.gui.server.ServerWindow(
 			notebookinfo,
 			public=True,
-			**self.get_options('template', 'port')
+			template=template,
+			port=port
 		)
 		window.show_all()
 		return window
@@ -421,7 +437,8 @@ class ExportCommand(NotebookCommand):
 	)
 
 	def get_exporter(self, page):
-		from zim.fs import File, Dir
+		from zim.newfs import localFileOrFolder, FilePath, LocalFile, LocalFolder, FileNotFoundError
+		from zim.templates import get_template
 		from zim.export import \
 			build_mhtml_file_exporter, \
 			build_single_file_exporter, \
@@ -429,26 +446,33 @@ class ExportCommand(NotebookCommand):
 			build_notebook_exporter
 
 		format = self.opts.get('format', 'html')
+		template = get_template(format, self.opts.get('template', 'Default'), pwd=self.pwd)
+
 		if not 'output' in self.opts:
 			raise UsageError(_('Output location needed for export')) # T: error in export command
-		output = Dir(self.opts['output'])
-		if not output.isdir():
-			output = File(self.opts.get('output'))
-		template = self.opts.get('template', 'Default')
 
-		if output.exists() and not self.opts.get('overwrite'):
-			if output.isdir():
-				if len(output.list()) > 0:
-					raise Error(_('Output folder exists and not empty, specify "--overwrite" to force export'))  # T: error message for export
+		try:
+			output = localFileOrFolder(self.opts['output'], pwd=self.pwd)
+		except FileNotFoundError:
+			# resolve path, but type undecided
+			output = FilePath(self.pwd).get_abspath(self.opts['output']) # can raise again for mal-formed paths
+		else:
+			# file or folder exists
+			if not self.opts.get('overwrite'):
+				if isinstance(output, LocalFolder):
+					if len(output.list_names()) > 0:
+						raise Error(_('Output folder exists and not empty, specify "--overwrite" to force export'))  # T: error message for export
+					else:
+						pass
 				else:
-					pass
-			else:
-				raise Error(_('Output file exists, specify "--overwrite" to force export'))  # T: error message for export
+					raise Error(_('Output file exists, specify "--overwrite" to force export'))  # T: error message for export
 
 		if format == 'mhtml':
 			self.ignore_options('index-page')
-			if output.isdir():
+			if isinstance(output, LocalFolder): # implies exists
 				raise UsageError(_('Need output file to export MHTML')) # T: error message for export
+			else:
+				output = LocalFile(output) # FilePath --> LocalFile
 
 			exporter = build_mhtml_file_exporter(
 				output, template,
@@ -456,9 +480,11 @@ class ExportCommand(NotebookCommand):
 			)
 		elif self.opts.get('singlefile'):
 			self.ignore_options('index-page')
-			if output.exists() and output.isdir():
+			if isinstance(output, LocalFolder):
 				ext = get_format(format).info['extension']
 				output = output.file(page.basename) + '.' + ext
+			else:
+				output = LocalFile(output) # FilePath --> LocalFile
 
 			exporter = build_single_file_exporter(
 				output, format, template, namespace=page,
@@ -466,19 +492,21 @@ class ExportCommand(NotebookCommand):
 			)
 		elif page:
 			self.ignore_options('index-page')
-			if output.exists() and output.isdir():
+			if isinstance(output, LocalFolder):
 				ext = get_format(format).info['extension']
 				output = output.file(page.basename) + '.' + ext
+			else:
+				output = LocalFile(output) # FilePath --> LocalFile
 
 			exporter = build_page_exporter(
 				output, format, template, page,
 				document_root_url=self.opts.get('root-url'),
 			)
 		else:
-			if not output.exists():
-				output = Dir(output.path)
-			elif not output.isdir():
+			if isinstance(output, LocalFile): # implies exists
 				raise UsageError(_('Need output folder to export full notebook')) # T: error message for export
+			else:
+				output = LocalFolder(output) # FilePath --> LocalFolder
 
 			exporter = build_notebook_exporter(
 				output, format, template,
@@ -491,14 +519,17 @@ class ExportCommand(NotebookCommand):
 	def run(self):
 		from zim.export.selections import AllPages, SinglePage, SubPages
 
-		notebook, page = self.build_notebook()
+		notebook, href = self.build_notebook()
 		notebook.index.check_and_update()
 
-		if page and self.opts.get('recursive'):
+		if href and self.opts.get('recursive'):
+			page = Path(href.names) # ignore anchor
 			selection = SubPages(notebook, page)
-		elif page:
+		elif href:
+			page = Path(href.names) # ignore anchor
 			selection = SinglePage(notebook, page)
 		else:
+			page = None
 			selection = AllPages(notebook)
 
 		exporter = self.get_exporter(page)
@@ -514,7 +545,7 @@ class SearchCommand(NotebookCommand):
 	def run(self):
 		from zim.search import SearchSelection, Query
 
-		notebook, p = self.build_notebook()
+		notebook, x = self.build_notebook()
 		n, query = self.get_arguments()
 
 		if query and not query.isspace():
@@ -533,13 +564,33 @@ class IndexCommand(NotebookCommand):
 	'''Class implementing the C{--index} command'''
 
 	arguments = ('NOTEBOOK',)
+	options = (
+		('flush', 'f', 'flush the index first and force re-building'),
+	)
 
 	def run(self):
-		notebook, p = self.build_notebook(ensure_uptodate=False)
-		notebook.index.flush()
-		for info in notebook.index.update_iter():
-			#logger.info('Indexing %s', info)
-			pass # TODO meaningful info for above message
+		# Elevate logging level of indexer to ensure "zim --index -V" gives
+		# some meaningfull output
+		def elevate_index_logging(log_record):
+			if log_record.levelno == logging.DEBUG:
+				log_record.levelno = logging.INFO
+				log_record.levelname = 'INFO'
+			return True
+
+		mylogger = logging.getLogger('zim.notebook.index')
+		mylogger.setLevel(logging.DEBUG)
+		mylogger.addFilter(elevate_index_logging)
+
+		notebook, x = self.build_notebook(ensure_uptodate=False)
+		if self.opts.get('flush'):
+			notebook.index.flush()
+			notebook.index.update()
+		else:
+			# Effectively the same as check_and_update_index ui action
+			logger.info('Checking notebook index')
+			notebook.index.check_and_update()
+
+		logger.info('Index up to date!')
 
 
 commands = {
@@ -578,6 +629,8 @@ def build_command(args, pwd=None):
 				# Can't use following because log level not yet set:
 				# logger.debug('Error while loading: zim.plugins.%s.Command', cmd, exc_info=sys.exc_info())
 			raise UsageError('Could not load commandline command for plugin "%s"' % cmd)
+		if klass is None:
+			raise UsageError('Module %s has no commandline command' % cmd)
 	else:
 		if args and args[0].startswith('--') and args[0][2:] in commands:
 			cmd = args.pop(0)[2:]
@@ -684,7 +737,8 @@ class ZimApplication(object):
 		self._run_cmd(cmd, args) # test seam
 
 	def _run_cmd(self, cmd, args):
-		self._setup_logging(cmd)
+		if not self._log_started:
+			self._log_start()
 
 		if self._running:
 			# This is not the first command that we process
@@ -771,21 +825,6 @@ class ZimApplication(object):
 
 		# exit immediatly if no toplevel created
 
-	def _setup_logging(self, cmd):
-		if cmd.opts.get('debug'):
-			level = logging.DEBUG
-		elif cmd.opts.get('verbose'):
-			level = logging.INFO
-		else:
-			level = logging.WARN
-
-		root = logging.getLogger() # root
-		if level < root.getEffectiveLevel():
-			root.setLevel(level)
-
-		if not self._log_started:
-			self._log_start()
-
 	def _log_start(self):
 		self._log_started = True
 
@@ -846,7 +885,7 @@ class ZimApplication(object):
 	def _daemonize(self):
 		# Decouple from parent environment
 		# and redirect standard file descriptors
-		os.chdir(zim.fs.Dir('~').path)
+		os.chdir(zim.newfs.LocalFolder('~').path)
 			# Using HOME because this folder will not disappear normally
 			# and because it is a sane starting point for file choosers etc.
 
@@ -863,8 +902,9 @@ class ZimApplication(object):
 			pass
 		else:
 			# Redirect output to file
-			dir = zim.fs.get_tmpdir()
-			err_stream = open(os.path.join(dir.path, "zim.log"), "w")
+			dir = zim.newfs.get_tmpdir()
+			zim.debug_log_file = os.path.join(dir.path, "zim.log")
+			err_stream = open(zim.debug_log_file, "w")
 
 			# Try to flush standards out and error, if there
 			for pipe in (sys.stdout, sys.stderr):
@@ -885,8 +925,8 @@ class ZimApplication(object):
 
 			# Re-initialize logging handler, in case it keeps reference
 			# to the old stderr object
+			rootlogger = logging.getLogger()
 			try:
-				rootlogger = logging.getLogger()
 				for handler in rootlogger.handlers:
 					rootlogger.removeHandler(handler)
 
@@ -896,6 +936,11 @@ class ZimApplication(object):
 			except:
 				pass
 
+			if rootlogger.getEffectiveLevel() != logging.DEBUG:
+				rootlogger.setLevel(logging.DEBUG) # else file remains empty
+				self._log_start()
+
+
 
 ZIM_APPLICATION = ZimApplication() # Singleton per process
 
@@ -914,7 +959,7 @@ def main(*argv):
 			'ERROR: Could not find data files in path: \n'
 			'%s\n'
 			'Try setting XDG_DATA_DIRS'
-				% list(map(str, zim.config.data_dirs()))
+				% list(map(str, zim.config.data_dirs(include_non_existing=True)))
 		)
 
 	try:
