@@ -16,7 +16,7 @@ logger = logging.getLogger('zim.newfs')
 from . import FS_SUPPORT_NON_LOCAL_FILE_SHARES
 
 from zim.errors import Error
-from zim.parsing import url_encode
+from zim.parsing import url_encode, url_decode
 
 
 is_url_re = re.compile('^\w{2,}:/')
@@ -107,12 +107,12 @@ def _split_file_url(url):
 	if path.startswith('/localhost/'): # exact 2 '/' before 'localhost'
 		path = path[11:]
 		isshare = False
-	elif scheme == 'smb' or re.match('^/\w', path): # exact 2 '/' before 'localhost'
+	elif scheme == 'smb' or re.match('^/\w', path): # exact 2 '/' followed by hostname
 		isshare = True
 	else:
 		isshare = False # either 'file:/' or 'file:///'
 
-	return path.strip('/').split('/'), isshare
+	return url_decode(path).strip('/').split('/'), isshare
 
 
 def _splitnormpath(path, force_rel=False):
@@ -143,7 +143,10 @@ def _splitnormpath(path, force_rel=False):
 		if name == '.' and names:
 			pass
 		elif name == '..':
-			if names and names[-1] != '..':
+			if names and names[-1] == '.':
+				names[-1] = name # e.g. "./../foo" --> ['..', 'foo']
+				makeroot = False
+			elif names and names[-1] != '..':
 				names.pop()
 			else:
 				names.append(name)
@@ -162,10 +165,11 @@ def _splitnormpath(path, force_rel=False):
 
 
 if os.name == 'nt':
-	def _joinabspath(names):
+	def _joinabspath(names, origpath=None):
+		# "origpath" is only used for better readable error message
 		# first element must be either drive letter or UNC host
 		if not re.match(r'^(\w:|\\\\\w)', names[0]):
-			raise ValueError('Not an absolute path: %s' % '\\'.join(names))
+			raise ValueError('Not an absolute path: %s' % (origpath or '\\'.join(names)))
 		else:
 			return '\\'.join(names) # Don't rely on SEP here, msys sets it to '/'
 
@@ -179,13 +183,14 @@ if os.name == 'nt':
 			return 'file://' + url_encode(names[0].strip('\\') + '/' + '/'.join(names[1:]))
 
 else:
-	def _joinabspath(names):
+	def _joinabspath(names, origpath=None):
+		# "origpath" is only used for better readable error message
 		if names[0].startswith('\\\\'):
 			return '\\'.join(names) # Windows share drive
 		elif names[0].startswith('/'):
 			return '/'.join(names)
 		else:
-			raise ValueError('Not an absolute path: %s' % '/'.join(names))
+			raise ValueError('Not an absolute path: %s' % (origpath or '/'.join(names)))
 
 	def _joinuri(names):
 		if names[0][0] == '/':
@@ -195,20 +200,31 @@ else:
 
 
 def _os_expanduser(path):
+	# Force usage of $HOME (especially on windows) instead of default logic
+	# in os.path.expanduser
+	# This depends on us setting HOME correctly based on USERPROFILE or similar
 	assert path.startswith('~')
-	path = os.path.expanduser(path)
+	home = os.environ['HOME']
+	parts = path.replace('\\', '/').strip('/').split('/')
+	if parts[0] == '~':
+		return SEP.join([home] + parts[1:])
+	else: # ~user
+		path = os.path.expanduser(path)
+		if path.startswith('~'):
+			# fallback
+			homedir = os.path.dirname(home)
+			return SEP.join([homedir, parts[0][1:]] + parts[1:])
+		else:
+			return path
 
-	if path.startswith('~'):
-		# expansion failed - do a simple fallback
-		home = os.environ['HOME']
-		parts = path.replace('\\', '/').strip('/').split('/')
-		if parts[0] == '~':
-			path = SEP.join([home] + parts[1:])
-		else: # ~user
-			dir = os.path.dirname(home) # /home or similar ?
-			path = SEP.join([dir, parts[0][1:]] + parts[1:])
 
-	return path
+def is_abs_filepath(string):
+	try:
+		_joinabspath(_splitnormpath(string), origpath=string)
+	except ValueError:
+		return False
+	else:
+		return True
 
 
 class FilePath(object):
@@ -230,7 +246,7 @@ class FilePath(object):
 	def __init__(self, path):
 		if isinstance(path, (tuple, list, str)):
 			self.pathnames = _splitnormpath(path)
-			self.path = _joinabspath(self.pathnames)
+			self.path = _joinabspath(self.pathnames, origpath=path)
 		elif isinstance(path, FilePath):
 			self.pathnames = path.pathnames
 			self.path = path.path
@@ -281,6 +297,16 @@ class FilePath(object):
 		else:
 			return self.path
 
+	def parent(self):
+		dirname = self.dirname
+		return FilePath(dirname) if dirname else None
+
+	def parents(self):
+		parent = self.parent()
+		while parent:
+			yield parent
+			parent = parent.parent()
+
 	def get_childpath(self, path):
 		assert path
 		names = _splitnormpath(path, force_rel=True)
@@ -404,15 +430,17 @@ class FSObjectBase(FilePath, metaclass=FSObjectMeta):
 		logger.debug('Cross FS type move %s --> %s', (self, other))
 		self._copyto(other)
 		self.remove()
+		return other
 
 	def remove(self, cleanup=True):
 		raise NotImplementedError
 
 	def _cleanup(self):
-		try:
-			self.parent().remove()
-		except (ValueError, FolderNotEmptyError):
-			pass
+		for parent in self.parents():
+			try:
+				parent.remove()
+			except FolderNotEmptyError:
+				break
 
 
 class Folder(FSObjectBase):
@@ -541,50 +569,75 @@ xdgmime = None
 mimetypes = None
 try:
 	import xdg.Mime as xdgmime
+	mytype = xdgmime.get_type('image.png', name_pri=80)
+	if str(mytype) != 'image/png':
+		# Even if xdg is installed, the database is not (always) initialized
+		logger.debug("Found 'xdg.Mime', but no database - falling back to 'mimetypes'")
+		xdgmime = None
+		import mimetypes
 except ImportError:
-	if os.name != 'nt':
-		logger.info("Can not import 'xdg.Mime' - falling back to 'mimetypes'")
-	else:
-		pass # Ignore this error on Windows; doesn't come with xdg.Mime
+	logger.debug("Can not import 'xdg.Mime' - falling back to 'mimetypes'")
 	import mimetypes
 
-
 #: Extensions to determine image mimetypes - used in L{File.isimage()}
-IMAGE_EXTENSIONS = (
+IMAGE_EXTENSIONS = {
 	# Gleaned from Gdk.get_formats()
-	'bmp', # image/bmp
-	'gif', # image/gif
-	'icns', # image/x-icns
-	'ico', # image/x-icon
-	'cur', # image/x-icon
-	'jp2', # image/jp2
-	'jpc', # image/jp2
-	'jpx', # image/jp2
-	'j2k', # image/jp2
-	'jpf', # image/jp2
-	'jpeg', # image/jpeg
-	'jpe', # image/jpeg
-	'jpg', # image/jpeg
-	'pcx', # image/x-pcx
-	'png', # image/png
-	'pnm', # image/x-portable-anymap
-	'pbm', # image/x-portable-anymap
-	'pgm', # image/x-portable-anymap
-	'ppm', # image/x-portable-anymap
-	'ras', # image/x-cmu-raster
-	'tga', # image/x-tga
-	'targa', # image/x-tga
-	'tiff', # image/tiff
-	'tif', # image/tiff
-	'wbmp', # image/vnd.wap.wbmp
-	'xbm', # image/x-xbitmap
-	'xpm', # image/x-xpixmap
-	'wmf', # image/x-wmf
-	'apm', # image/x-wmf
-	'svg', # image/svg+xml
-	'svgz', # image/svg+xml
-	'svg.gz', # image/svg+xml
-)
+	'bmp': 'image/bmp',
+	'gif': 'image/gif',
+	'icns': 'image/x-icns',
+	'ico': 'image/x-icon',
+	'cur': 'image/x-icon',
+	'jp2': 'image/jp2',
+	'jpc': 'image/jp2',
+	'jpx': 'image/jp2',
+	'j2k': 'image/jp2',
+	'jpf': 'image/jp2',
+	'jpeg': 'image/jpeg',
+	'jpe': 'image/jpeg',
+	'jpg': 'image/jpeg',
+	'pcx': 'image/x-pcx',
+	'png': 'image/png',
+	'pnm': 'image/x-portable-anymap',
+	'pbm': 'image/x-portable-anymap',
+	'pgm': 'image/x-portable-anymap',
+	'ppm': 'image/x-portable-anymap',
+	'ras': 'image/x-cmu-raster',
+	'tga': 'image/x-tga',
+	'targa': 'image/x-tga',
+	'tiff': 'image/tiff',
+	'tif': 'image/tiff',
+	'wbmp': 'image/vnd.wap.wbmp',
+	'xbm': 'image/x-xbitmap',
+	'xpm': 'image/x-xpixmap',
+	'wmf': 'image/x-wmf',
+	'apm': 'image/x-wmf',
+	'svg': 'image/svg+xml',
+	'svgz': 'image/svg+xml',
+	'svg.gz': 'image/svg+xml',
+	# Custom additions
+	'webp': 'image/webp',
+}
+
+
+def get_mimetype_from_path(path):
+	if '.' in path:
+		_, ext = path.rsplit('.', 1)
+		if ext.lower() in IMAGE_EXTENSIONS:
+			return IMAGE_EXTENSIONS[ext.lower()]
+
+	if xdgmime:
+		mimetype = xdgmime.get_type(path, name_pri=80)
+		return str(mimetype)
+	else:
+		mimetype, encoding = mimetypes.guess_type(path, strict=False)
+		if encoding == 'gzip':
+			return 'application/x-gzip'
+		elif encoding == 'bzip2':
+			return 'application/x-bzip2'
+		elif encoding == 'compress':
+			return 'application/x-compress'
+		else:
+			return mimetype or 'application/octet-stream'
 
 
 def _md5(content):
@@ -633,26 +686,17 @@ class File(FSObjectBase):
 		@returns: the mimetype as a string, e.g. "text/plain"
 		'''
 		if self._mimetype is None:
-			if xdgmime:
-				mimetype = xdgmime.get_type(self.path, name_pri=80)
-				self._mimetype = str(mimetype)
-			else:
-				mimetype, encoding = mimetypes.guess_type(self.path, strict=False)
-				if encoding == 'gzip':
-					return 'application/x-gzip'
-				elif encoding == 'bzip2':
-					return 'application/x-bzip2'
-				elif encoding == 'compress':
-					return 'application/x-compress'
-				else:
-					self._mimetype = mimetype or 'application/octet-stream'
+			self._mimetype = get_mimetype_from_path(self.path)
 
 		return self._mimetype
 
 	def size(self):
 		raise NotImplementedError
 
-	def read(self):
+	def read(self, size=-1):
+		raise NotImplementedError
+
+	def readline(self, size=-1):
 		raise NotImplementedError
 
 	def readlines(self):
