@@ -38,11 +38,7 @@ import logging
 import inspect
 import weakref
 
-try:
-	import collections.abc as abc
-except ImportError:
-	# python < version 3.3
-	import collections as abc
+import collections.abc as abc
 
 from zim.newfs import LocalFolder, LocalFile
 
@@ -70,9 +66,9 @@ logger = logging.getLogger('zim.plugins')
 # Also this switch makes it easier to have a single instruction for
 # users where to put custom plugins.
 
-PLUGIN_FOLDER = XDG_DATA_HOME.subdir('zim/plugins')
+PLUGIN_FOLDER = XDG_DATA_HOME.folder('zim/plugins')
 
-for dir in data_dirs('plugins'):
+for dir in data_dirs('plugins', include_non_existing=True):
 	__path__.append(dir.path)
 
 __path__.append(__path__.pop(0)) # reshuffle real module path to the end
@@ -717,7 +713,9 @@ class PluginClass(ConnectorMixin):
 			else:
 				key, type, label, default, check = pref
 
-			if type in ('int', 'choice'):
+			if label is None:
+				pass # Hidden options
+			elif type in ('int', 'choice'):
 				fields.append((key, type, label, check))
 			else:
 				fields.append((key, type, label))
