@@ -17,7 +17,7 @@ from zim.notebook.index.base import TreeModelMixinBase
 from zim.notebook.index.pages import PagesTreeModelMixin, PageIndexRecord, IndexNotFoundError, IS_PAGE
 
 from zim.plugins import PluginClass
-from zim.actions import PRIMARY_MODIFIER_MASK
+from zim.actions import PRIMARY_MODIFIER_MASK, action
 
 from zim.gui.notebookview import NotebookViewExtension
 from zim.gui.widgets import BrowserTreeView, ScrolledWindow, \
@@ -126,10 +126,25 @@ class PageIndexNotebookViewExtension(NotebookViewExtension):
 		model = PageTreeStore(self.pageview.notebook.index)
 		self.treeview.set_model(model)
 
+	@action(_('Collapse Page Index'), accelerator=None, menuhints='view') # T: menu item
+	def collapse(self):
+		self.treeview.collapse_all()
+
+	@action(_('Reveal Current Page in Page Index'), accelerator=None, menuhints='view') # T: menu item
+	def reveal(self):
+		model = self.treeview.get_model()
+		try:
+			treepaths = model.find_all(model.current_page)
+		except IndexNotFoundError:
+			return
+		else:
+			if len(treepaths) != 0:
+				self.treeview.select_treepath(treepaths[0])
+
 
 class PageIndexWidget(Gtk.VBox, WindowSidePaneWidget):
 
-	title = _('Index')	# T: tab label for side pane
+	title = _('Inde_x')	# T: tab label for side pane
 
 	def __init__(self, treeview):
 		GObject.GObject.__init__(self)
@@ -539,7 +554,7 @@ class PageTreeView(BrowserTreeView):
 		assert selectiondata.get_target().name() == PAGELIST_TARGET_NAME
 		data = selectiondata.get_data()
 		logger.debug('Drag data recieved: %r', data)
-		if data is None:
+		if data is None or len(data) == 0:
 			data = zim.gui.clipboard._internal_selection_data # HACK issue #390
 			zim.gui.clipboard._internal_selection_data = None
 			logger.debug('Got data via workaround: %s', data)
